@@ -32,7 +32,7 @@ from fresco.util.urls import url_join
 from fresco.util.http import parse_header
 from fresco.util.wsgi import unicode_to_environ
 
-__version__ = '7.4.5'
+__version__ = '7.4.6'
 
 #: Registry for xpath multimethods
 xpath_registry = {}
@@ -540,17 +540,31 @@ class ElementWrapper(object):
         Fill the current form with data.
 
         :param \*args: Pairs of ``(selector, value)``
-
         :param \*\*kwargs: mappings of fieldname to value
+        :param _fill_strict: If True, raise an error when a field is not found
 
         See the documentation for :meth:`_set_value` implementations
         for individual form control types to see how values are processed
         as this varies between text inputs, selects, radio buttons,
         checkboxes etc
         """
+        strict = kwargs.pop('_fill_strict', True)
+
+        def check_exists(element, name):
+            if len(element) > 0:
+                return True
+
+            if strict:
+                valid = ', '.join(e.name
+                                  for e in self.css('input, textarea, select'))
+                raise IndexError(
+                    "Couldn't find a form element named {0!r}. "
+                    "Valid names are {1}".format(name, valid))
 
         for selector, value in args:
-            self(selector).fill(value)
+            element = self(selector)
+            if check_exists(element, selector):
+                element.fill(value)
 
         for name, value in kwargs.items():
             path = ".//*[(local-name() = 'input' "\
@@ -558,16 +572,16 @@ class ElementWrapper(object):
                    "or local-name() = 'select') "\
                    "and (@name=$name or @id=$name)]"
             element = self.find(path, name=name)
-            if len(element) == 0:
-                fields = ', '.join(e.name
-                                   for e in self('css, textarea, select'))
-                raise IndexError(
-                    "Couldn't find a form element named {0!r}. "
-                    "Valid names are {1}".format(name, fields))
-            element.fill(value)
+            if check_exists(element, name):
+                element.fill(value)
 
         return self
 
+    @when("form")
+    def fill_sloppy(self, *args, **kwargs):
+        kwargs['_fill_strict'] = False
+        return self.fill(*args, **kwargs)
+
     @when("input[@type='checkbox']")  # NOQA
     def fill(self, values):
 
@@ -1703,10 +1717,11 @@ class PassStateWSGIApp(object):
         return self.first_request(environ, start_response)
 
     def redirect_to_initial_path(self, environ, start_response):
+        path = unicode_to_environ(self.initial_path)
+        path = quote(path)
         start_response('302 Found', [('Content-Type', 'text/html'),
-                                     ('Location', self.initial_path)])
-        return ['<a href="{0}">{0}</a>'.format(self.initial_path)
-                                       .encode('utf8')]
+                                     ('Location', path)])
+        return [u'<a href="{0}">{0}</a>'.format(path).encode('utf8')]
 
     def first_request(self, environ, start_response):
         self.first_request_served = True
@@ -1761,8 +1776,9 @@ def url_join_same_server(baseurl, url):
     baseurl = urlparse(baseurl)
     if normalize_host(baseurl.scheme, baseurl.netloc) != \
             normalize_host(url.scheme, url.netloc):
-        raise ValueError("URI links to another server: %s" %
-                         (urlunparse(url),))
+        raise ValueError("URI links to another server: %s (expected %s)" %
+                         (urlunparse(url),
+                          normalize_host(baseurl.scheme, baseurl.netloc)))
     return urlunparse(('', '') + url[2:])
 
 
